package com.lmert.util;

import java.net.Socket;
import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.net.HttpURLConnection;
import java.util.Map;
import java.nio.charset.*;

class ClientConn implements Runnable {

   InputStream in;
   PrintStream out;
   boolean quit_flag;
   Exception init_ex;

   ClientConn(Socket sock) {
      try {
	 in = sock.getInputStream();
	 out = new PrintStream(sock.getOutputStream());
      } catch (IOException e) {
	 init_ex = e;
      }
   }

   public void run() {
      if (init_ex != null)
	 throw new RuntimeException("Exception before thread started",init_ex);
      try {
	 String line;

	 while (!quit_flag && (line=readLine())!=null) {
	    String[] arr = line.split(" ");
	    if (arr.length != 3 ||
		!arr[2].matches("HTTP/1[.][01]")) {
	       sendBadRequest(); return;
	    }
	       
	    String cmd = arr[0].toUpperCase();

	    if ((cmd.equals("GET")||cmd.equals("HEAD"))&&
	        (arr[1].startsWith("http://")||arr[1].startsWith("https://"))) 
	    {
	       URL url = new URL(arr[1]);
	       
	       for ( ; (line=readLine())!= null && !line.equals(""); ) {
			       // TODO:  parse header lines
	       }
	       
	       HttpURLConnection htconn 
		  = (HttpURLConnection) url.openConnection();
 
	       htconn.setRequestMethod(cmd);
	       
	       htconn.connect();
	       int resp = htconn.getResponseCode();
	       String msg = htconn.getResponseMessage();

	       out.print("HTTP/1.0 "+resp+" "+msg+"\r\n");
	       long dt = htconn.getDate();
	       long lm = htconn.getLastModified();
	       long len= htconn.getContentLength();

	       java.text.DateFormat df = new java.text.SimpleDateFormat(
		  "E, dd MMM yyyy HH:mm:ss ");
	       df.setTimeZone(java.util.TimeZone.getTimeZone("GMT+0:00"));

	       if (len!=-1)
		  out.print("Content-Length: "+len+"\r\n");
	       if (lm!=0)
		  out.print("Last-Modified: "+df.format(new java.util.Date(lm))
			    +" GMT\r\n");
	       if (dt==0) dt=System.currentTimeMillis();
	       out.print("Date: "+df.format(new java.util.Date(dt))
			 +" GMT\r\n");

	       out.print("\r\n");

	       java.io.InputStream uis = htconn.getInputStream();
	       byte[] buf;

	       if (len!=-1) {
		  buf = new byte[(int)len];
		  uis.read(buf);
		  out.write(buf);
	       } else {
		  int nread;
		  buf = new byte[1000];
		  while ((nread=uis.read(buf))>0)
		     out.write(buf,0,nread);
		  out.close();
		  quit_flag = true;
		  in.close();
	       }
	    }
	    

	 }
      } catch (IOException e) {
	 throw new RuntimeException(e);
      }
   }

   void sendBadRequest() {
      out.println("HTTP/1.0 400 Bad Request\r\n\r\n");
      out.flush();
      out.close();
   }

   String readLine() throws IOException {
      StringBuffer buf = new StringBuffer();
      byte[] bb = new byte[1];
      boolean r_seen = false;
      boolean has8bit = false;

      try {
	 in.mark(1);
	 while (in.read(bb)==1) {
	    if (bb[0]=='\n') break;
	    if (r_seen) {
	       in.reset();
	    }
	    in.mark(1);
	    if (bb[0]=='\r') {
	       r_seen=true;
	       continue;
	    }
	    if (bb[0]==0) {}
	    else {
	       if (bb[0] < 0) has8bit = true;
	       buf.append((char)bb[0]);
	    }
	 }
      } catch (EOFException e) {
	 if (buf.length()==0) return null;
      }
      
      if (has8bit) {
	 try {
	    return Charset.forName("UTF-8").newDecoder().decode(java.nio.ByteBuffer.wrap(buf.toString().getBytes("ISO8859-1"))).toString();
	 } catch (CharacterCodingException e) {}
      }
      return buf.toString();
   }

}
